@@ -3,6 +3,7 @@ hledger-ui - a hledger add-on providing a curses-style interface.
 Copyright (c) 2007-2015 Simon Michael <simon@joyful.com>
 Released under GPL version 3 or later.
 -}
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE TemplateHaskell #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
@@ -11,32 +12,53 @@ module Hledger.UI.Main where
 
 -- import Control.Applicative
 -- import Lens.Micro.Platform ((^.))
+import Control.Concurrent (threadDelay)
+import Control.Concurrent.Async
 import Control.Monad
 -- import Control.Monad.IO.Class (liftIO)
--- import Data.Default
--- import Data.Monoid              -- 
+#if !MIN_VERSION_vty(0,15,0)
+import Data.Default (def)
+#endif
+-- import Data.Monoid              --
 import Data.List
 import Data.Maybe
 -- import Data.Text (Text)
 import qualified Data.Text as T
 -- import Data.Time.Calendar
+import Graphics.Vty (mkVty)
 import Safe
 import System.Exit
-
-import qualified Graphics.Vty as V
+import System.Directory
+import System.FilePath
+import System.FSNotify
 import Brick
 
+#if MIN_VERSION_brick(0,16,0)
+import qualified Brick.BChan as BC
+#else
+import Control.Concurrent.Chan (newChan, writeChan)
+#endif
+
 import Hledger
 import Hledger.Cli hiding (progname,prognameandversion,green)
 import Hledger.UI.UIOptions
 import Hledger.UI.UITypes
--- import Hledger.UI.UIUtils
+import Hledger.UI.UIState (toggleHistorical)
 import Hledger.UI.Theme
 import Hledger.UI.AccountsScreen
 import Hledger.UI.RegisterScreen
 
 ----------------------------------------------------------------------
 
+#if MIN_VERSION_brick(0,16,0)
+newChan :: IO (BC.BChan a)
+newChan = BC.newBChan 10
+
+writeChan :: BC.BChan a -> a -> IO ()
+writeChan = BC.writeBChan
+#endif
+
+
 main :: IO ()
 main = do
   opts <- getHledgerUIOpts
@@ -125,25 +147,94 @@ runBrickUi uopts@UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}} j = do
                    ,aPrevScreens=[]
                    ,aMode=Normal
                    }
-  
-    ui = (sInit scr) d True
-         UIState{
-            aopts=uopts'
-           ,ajournal=j
-           ,aScreen=scr
-           ,aPrevScreens=prevscrs
-           ,aMode=Normal
-           }
 
-    brickapp :: App (UIState) V.Event Name
+    ui =
+      (sInit scr) d True $
+        (if change_ uopts' then toggleHistorical else id) $ -- XXX
+        UIState{
+          aopts=uopts'
+         ,ajournal=j
+         ,aScreen=scr
+         ,aPrevScreens=prevscrs
+         ,aMode=Normal
+         }
+
+    brickapp :: App (UIState) AppEvent Name
     brickapp = App {
-        appLiftVtyEvent = id
-      , appStartEvent   = return
+        appStartEvent   = return
       , appAttrMap      = const theme
       , appChooseCursor = showFirstCursor
       , appHandleEvent  = \ui ev -> sHandle (aScreen ui) ui ev
       , appDraw         = \ui    -> sDraw   (aScreen ui) ui
       }
 
-  void $ defaultMain brickapp ui
+  if not (watch_ uopts')
+  then
+    void $ defaultMain brickapp ui
 
+  else do
+    -- a channel for sending misc. events to the app
+    eventChan <- newChan
+
+    -- start a background thread reporting changes in the current date
+    -- use async for proper child termination in GHCI
+    let
+      watchDate old = do
+        threadDelay 1000000 -- 1 s
+        new <- getCurrentDay
+        when (new /= old) $ do
+          let dc = DateChange old new
+          -- dbg1IO "datechange" dc -- XXX don't uncomment until dbg*IO fixed to use traceIO, GHC may block/end thread
+          -- traceIO $ show dc
+          writeChan eventChan dc
+        watchDate new
+
+    withAsync
+      (getCurrentDay >>= watchDate)
+      $ \_ -> do
+
+      -- start one or more background threads reporting changes in the directories of our files
+      -- XXX many quick successive saves causes the problems listed in BUGS
+      -- with Debounce increased to 1s it easily gets stuck on an error or blank screen
+      -- until you press g, but it becomes responsive again quickly.
+      -- withManagerConf defaultConfig{confDebounce=Debounce 1} $ \mgr -> do
+      -- with Debounce at the default 1ms it clears transient errors itself
+      -- but gets tied up for ages
+      withManager $ \mgr -> do
+        dbg1IO "fsnotify using polling ?" $ isPollingManager mgr
+        files <- mapM canonicalizePath $ map fst $ jfiles j
+        let directories = nub $ sort $ map takeDirectory files
+        dbg1IO "files" files
+        dbg1IO "directories to watch" directories
+
+        forM_ directories $ \d -> watchDir
+          mgr
+          d
+          -- predicate: ignore changes not involving our files
+          (\fev -> case fev of
+            Modified f _ -> f `elem` files
+            -- Added    f _ -> f `elem` files
+            -- Removed  f _ -> f `elem` files
+            -- we don't handle adding/removing journal files right now
+            -- and there might be some of those events from tmp files
+            -- clogging things up so let's ignore them
+            _ -> False
+            )
+          -- action: send event to app
+          (\fev -> do
+            -- return $ dbglog "fsnotify" $ showFSNEvent fev -- not working
+            dbg1IO "fsnotify" $ showFSNEvent fev
+            writeChan eventChan FileChange
+            )
+
+        -- and start the app. Must be inside the withManager block
+#if MIN_VERSION_vty(0,15,0)
+        let myVty = mkVty mempty
+#else
+        let myVty = mkVty def
+#endif
+        void $ customMain myVty (Just eventChan) brickapp ui
+
+showFSNEvent (Added    f _) = "Added "    ++ show f
+showFSNEvent (Modified f _) = "Modified " ++ show f
+showFSNEvent (Removed  f _) = "Removed "  ++ show f
